/*  Project: Open Source Gmini Tools
    file: BreakpointList.cpp, Breakpoint management

    Copyright (C) 2004  The gmEmu Project <gmemu@donat.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
using namespace std;
#include "BreakpointList.h"
#include "Console.h"

void BreakpointList::BPX( u22 address ) {
   Breakpoint *BX = new BreakpointX( address );
   Breakpoints.push_back( BX );
   console.output( "added breakpoint for execution address 0x%x\n", address );
}
void BreakpointList::BPM( u22 address, unsigned int access ) {
   Breakpoint *BM = new BreakpointM( address, access );
   Breakpoints.push_back( BM );
   console.output( "added breakpoint for memory access address 0x%x\n", address );
}
void BreakpointList::BPR( u22 start, u22 end, unsigned int access ) {
   Breakpoint *BR = new BreakpointR( start, end, access );
   Breakpoints.push_back( BR );
   console.output( "added breakpoint for memory access in range 0x%x to 0x%x\n", start, end );
}
void BreakpointList::del( unsigned int brk ) {
   if( brk < Breakpoints.size(  ) ) {
      console.output( "deleted breakpoint " );
      Print( brk );
      Breakpoint *temp = Breakpoints[brk];
      Breakpoints.erase( Breakpoints.begin(  ) + brk );
      delete temp;
   }
}
void BreakpointList::clear(  ) {
   console.output( "deleted all breakpoints\n" );
   Breakpoints.clear(  );
}

void BreakpointList::Print(  ) {
   for( unsigned int i = 0; i < Breakpoints.size(  ); i++ )
      Print( i );
}
void BreakpointList::Print( unsigned int i ) {
  if (i < Breakpoints.size()) {
    console.output( "%2d - ", i );
    Breakpoints[i]->Print(  );
    console.output( "\n" );
  } else {
    console.output(" System breakpoint.\n");
  }
}

bool BreakpointList::isBreakpoint( u22 address, unsigned int access ) {
   if( breakPointEnabled ) {
      unsigned int max = Breakpoints.size(  );
      unsigned int i = 0;
      for( ; i < max && !Breakpoints[i]->is( address, access ); ++i ) ;
      if( i < max ) {
         breakPointEnabled = false;
         lastBreak = i;
         return true;
      }
   }
   return false;
}
