/*  Project: Open Source Gmini Tools
    file: LCDController.cpp, LCD controller emulation

    Copyright (C) 2004  The gmEmu Project <gmemu@donat.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Emulator.h"
#include <assert.h>
#include "util.h"
#include "Console.h"

#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>

LCDController::LCDController( Emulator * gm ) {
   gmini = gm;

   for (int i = 0; i< 0x40; i++)
     regs[i] = 0;
}

LCDController::~LCDController(  ) {
}

u8 LCDController::read8( u32 reg ) {
  u8 value = getReg8(reg);
  if (TRACE_LCD)
    gmini->tracer.log("LCD 8 %s -> %02x",MMIO::mapped[reg+MMIO::LCD_BASE], value);
  return value;
}

bool LCDController::write8( u32 reg, u8 value ) {
  if (TRACE_LCD)
    gmini->tracer.log("LCD 8 %s <- %02x", 
		      MMIO::mapped[reg+MMIO::LCD_BASE], value);
  setReg8(reg, value);
  return true;
}

u16 LCDController::read16( u32 reg ) {
  u16 value = getReg16(reg);
  if (TRACE_LCD)
    gmini->tracer.log("LCD 16 %s -> %02x",MMIO::mapped[reg+MMIO::LCD_BASE], value);
  return value;
}

bool LCDController::write16( u32 reg, u16 value ) {
  if (TRACE_LCD)
    gmini->tracer.log("LCD 16 %s <- %02x", 
		      MMIO::mapped[reg+MMIO::LCD_BASE], value);
  setReg16(reg, value);
  return true;
}

u8 LCDController::getReg8(int reg) {
  return regs[reg];
}
u16 LCDController::getReg16(int reg) {
  return (regs[reg] << 8) + regs[reg+1];
}
u24 LCDController::getReg24(int reg) {
  return (regs[reg] << 16) + (regs[reg+1] << 8) + regs[reg+2];
}

void LCDController::setReg8(int reg, u8 value) {
  regs[reg] = value;
}
void LCDController::setReg16(int reg, u16 value) {
  regs[reg+0] = value >> 8;
  regs[reg+1] = value;
}
void LCDController::setReg24(int reg, u24 value) {
  regs[reg+0] = value >> 16;
  regs[reg+1] = value >> 8;
  regs[reg+2] = value;
}


bool LCDController::enabled() {
  return getReg16(LCNTR0) & 0x01;
}

int LCDController::bpp() {
  return 1 << ((getReg16(LCNTR0) >> 5) & 0x03);
}

int LCDController::width() {
  return (getReg16(LCNTR1) & 0x1FF)+1;
}

int LCDController::height() {
  return (getReg16(LCNTR3) & 0x1FF)+1;
}

int LCDController::frameBuffer() {
  return (getReg24(LDBAR1) << 1) & 0xFFFFFF;
}
