/*  Project: Open Source Gmini Tools
    file: Reference.cpp, list of references to an address

    Copyright (C) 2004  The gmEmu Project <gmemu@donat.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <assert.h>
#include <stdio.h>
#include "Reference.h"
#include <ctype.h>

void parseParams( char *temp, vector < ParamDescr > &params ) {
   int len = strlen( temp );
   // parse parameters
   params.clear(  );
   int i = 0;
   while( i < len && temp[i] != '{' ) {
      i++;
   }
   if( temp[i] == '{' ) {
      // printf("Parsing parameters in %s\n", temp+i);
      while( i < len && temp[i] != '}' ) {
         i++;
         int ni = i;
         while( i < len && temp[i] != '=' )
            i++;
         ParamDescr p;
         p.name = string( temp ).substr( ni, i - ni );
         assert( i < len );     // '=' not found
         i++;
         switch ( tolower( temp[i] ) ) {
            case 'r':
               i++;
               if( temp[i] == 'x' ) {
                  i++;
                  p.loc.kind = Location::REG32;
               }
               else
                  p.loc.kind = Location::REG16;
               break;
            case 's':
               i++;
               if( temp[i] == 'x' ) {
                  i++;
                  p.loc.kind = Location::STACK32;
               }
               else
                  p.loc.kind = Location::STACK16;
               break;
            case 'a':
               i++;
               p.loc.kind = Location::REG22;
               break;
            default:
               assert( false );
         }
         ni = i;
         while( i < len && temp[i] != ',' && temp[i] != ':' && temp[i] != '}' ) {
            i++;
         }
         string idx = string( temp ).substr( ni, i - ni );
         p.loc.index = atoi( idx.c_str(  ) );
         if( temp[i] == ':' ) {
            i++;
            switch ( temp[i] ) {
               case 's':
                  p.format = ParamDescr::DSTR;
                  i++;
                  break;
               default:
                  assert( false );
            }
         }
         else
            p.format = ParamDescr::HEX;
         params.push_back( p );
         assert( i < len );
      }
      assert( temp[i] == '}' );
   }
}
const char Reference::typenames[4][6] = { "DATA", "RJUMP", "AJUMP", "CALL" };
Reference::Reference( char *sym ) {
   char *temp = new char[strlen( sym ) + 1];
   strcpy( temp, sym );
   name = temp;
   parseParams( temp, params );
}

Reference::~Reference(  ) {
   delete[] name;
}

void Reference::add( u32 from, access type ) {
   if( refs.find( from ) != refs.end(  ) && type != refs[from] )
      printf( "Redefining reference from %s to %s\n", typenames[refs[from]], typenames[type] );
   refs[from] = type;
}

#define CONST const             // bad indent
const char *Reference::getname(  ) CONST {
   return name;
};
const int Reference::getrefs( char *buf, bool html ) {
   map < u32, access >::const_iterator itr = refs.begin(  );
   int length = 0;
   int count = 0;
   if( itr == refs.end(  ) )
      return 0;
   length += sprintf( buf + length, "   " );
   while( itr != refs.end(  ) ) {
      if( html )
         length +=
           sprintf( buf + length, "<a href=\"#%06X\">0x%06X(%c)</a>",
                    itr->first, itr->first, typenames[itr->second][0] );
      else
         length += sprintf( buf + length, "%s 0x%06X\n", typenames[itr->second], itr->first );
      ++itr;
      ++count;
      if( html && itr != refs.end(  ) ) {

         if( count % 5 == 0 )
            length += sprintf( buf + length, ",\n   " );
         else
            length += sprintf( buf + length, ", " );
      }
   }
   return length;
}
